<?php

namespace Aws\CognitoIdentityProvider;

use Aws\AwsClient;
use Aws\Result;
use GuzzleHttp\Promise\Promise;

/**
 * This client is used to interact with the **Amazon Cognito Identity Provider** service.
 *
 * @method Result addCustomAttributes( array $args = [] )
 * @method Promise addCustomAttributesAsync( array $args = [] )
 * @method Result adminAddUserToGroup( array $args = [] )
 * @method Promise adminAddUserToGroupAsync( array $args = [] )
 * @method Result adminConfirmSignUp( array $args = [] )
 * @method Promise adminConfirmSignUpAsync( array $args = [] )
 * @method Result adminCreateUser( array $args = [] )
 * @method Promise adminCreateUserAsync( array $args = [] )
 * @method Result adminDeleteUser( array $args = [] )
 * @method Promise adminDeleteUserAsync( array $args = [] )
 * @method Result adminDeleteUserAttributes( array $args = [] )
 * @method Promise adminDeleteUserAttributesAsync( array $args = [] )
 * @method Result adminDisableProviderForUser( array $args = [] )
 * @method Promise adminDisableProviderForUserAsync( array $args = [] )
 * @method Result adminDisableUser( array $args = [] )
 * @method Promise adminDisableUserAsync( array $args = [] )
 * @method Result adminEnableUser( array $args = [] )
 * @method Promise adminEnableUserAsync( array $args = [] )
 * @method Result adminForgetDevice( array $args = [] )
 * @method Promise adminForgetDeviceAsync( array $args = [] )
 * @method Result adminGetDevice( array $args = [] )
 * @method Promise adminGetDeviceAsync( array $args = [] )
 * @method Result adminGetUser( array $args = [] )
 * @method Promise adminGetUserAsync( array $args = [] )
 * @method Result adminInitiateAuth( array $args = [] )
 * @method Promise adminInitiateAuthAsync( array $args = [] )
 * @method Result adminLinkProviderForUser( array $args = [] )
 * @method Promise adminLinkProviderForUserAsync( array $args = [] )
 * @method Result adminListDevices( array $args = [] )
 * @method Promise adminListDevicesAsync( array $args = [] )
 * @method Result adminListGroupsForUser( array $args = [] )
 * @method Promise adminListGroupsForUserAsync( array $args = [] )
 * @method Result adminListUserAuthEvents( array $args = [] )
 * @method Promise adminListUserAuthEventsAsync( array $args = [] )
 * @method Result adminRemoveUserFromGroup( array $args = [] )
 * @method Promise adminRemoveUserFromGroupAsync( array $args = [] )
 * @method Result adminResetUserPassword( array $args = [] )
 * @method Promise adminResetUserPasswordAsync( array $args = [] )
 * @method Result adminRespondToAuthChallenge( array $args = [] )
 * @method Promise adminRespondToAuthChallengeAsync( array $args = [] )
 * @method Result adminSetUserMFAPreference( array $args = [] )
 * @method Promise adminSetUserMFAPreferenceAsync( array $args = [] )
 * @method Result adminSetUserPassword( array $args = [] )
 * @method Promise adminSetUserPasswordAsync( array $args = [] )
 * @method Result adminSetUserSettings( array $args = [] )
 * @method Promise adminSetUserSettingsAsync( array $args = [] )
 * @method Result adminUpdateAuthEventFeedback( array $args = [] )
 * @method Promise adminUpdateAuthEventFeedbackAsync( array $args = [] )
 * @method Result adminUpdateDeviceStatus( array $args = [] )
 * @method Promise adminUpdateDeviceStatusAsync( array $args = [] )
 * @method Result adminUpdateUserAttributes( array $args = [] )
 * @method Promise adminUpdateUserAttributesAsync( array $args = [] )
 * @method Result adminUserGlobalSignOut( array $args = [] )
 * @method Promise adminUserGlobalSignOutAsync( array $args = [] )
 * @method Result associateSoftwareToken( array $args = [] )
 * @method Promise associateSoftwareTokenAsync( array $args = [] )
 * @method Result changePassword( array $args = [] )
 * @method Promise changePasswordAsync( array $args = [] )
 * @method Result confirmDevice( array $args = [] )
 * @method Promise confirmDeviceAsync( array $args = [] )
 * @method Result confirmForgotPassword( array $args = [] )
 * @method Promise confirmForgotPasswordAsync( array $args = [] )
 * @method Result confirmSignUp( array $args = [] )
 * @method Promise confirmSignUpAsync( array $args = [] )
 * @method Result createGroup( array $args = [] )
 * @method Promise createGroupAsync( array $args = [] )
 * @method Result createIdentityProvider( array $args = [] )
 * @method Promise createIdentityProviderAsync( array $args = [] )
 * @method Result createResourceServer( array $args = [] )
 * @method Promise createResourceServerAsync( array $args = [] )
 * @method Result createUserImportJob( array $args = [] )
 * @method Promise createUserImportJobAsync( array $args = [] )
 * @method Result createUserPool( array $args = [] )
 * @method Promise createUserPoolAsync( array $args = [] )
 * @method Result createUserPoolClient( array $args = [] )
 * @method Promise createUserPoolClientAsync( array $args = [] )
 * @method Result createUserPoolDomain( array $args = [] )
 * @method Promise createUserPoolDomainAsync( array $args = [] )
 * @method Result deleteGroup( array $args = [] )
 * @method Promise deleteGroupAsync( array $args = [] )
 * @method Result deleteIdentityProvider( array $args = [] )
 * @method Promise deleteIdentityProviderAsync( array $args = [] )
 * @method Result deleteResourceServer( array $args = [] )
 * @method Promise deleteResourceServerAsync( array $args = [] )
 * @method Result deleteUser( array $args = [] )
 * @method Promise deleteUserAsync( array $args = [] )
 * @method Result deleteUserAttributes( array $args = [] )
 * @method Promise deleteUserAttributesAsync( array $args = [] )
 * @method Result deleteUserPool( array $args = [] )
 * @method Promise deleteUserPoolAsync( array $args = [] )
 * @method Result deleteUserPoolClient( array $args = [] )
 * @method Promise deleteUserPoolClientAsync( array $args = [] )
 * @method Result deleteUserPoolDomain( array $args = [] )
 * @method Promise deleteUserPoolDomainAsync( array $args = [] )
 * @method Result describeIdentityProvider( array $args = [] )
 * @method Promise describeIdentityProviderAsync( array $args = [] )
 * @method Result describeResourceServer( array $args = [] )
 * @method Promise describeResourceServerAsync( array $args = [] )
 * @method Result describeRiskConfiguration( array $args = [] )
 * @method Promise describeRiskConfigurationAsync( array $args = [] )
 * @method Result describeUserImportJob( array $args = [] )
 * @method Promise describeUserImportJobAsync( array $args = [] )
 * @method Result describeUserPool( array $args = [] )
 * @method Promise describeUserPoolAsync( array $args = [] )
 * @method Result describeUserPoolClient( array $args = [] )
 * @method Promise describeUserPoolClientAsync( array $args = [] )
 * @method Result describeUserPoolDomain( array $args = [] )
 * @method Promise describeUserPoolDomainAsync( array $args = [] )
 * @method Result forgetDevice( array $args = [] )
 * @method Promise forgetDeviceAsync( array $args = [] )
 * @method Result forgotPassword( array $args = [] )
 * @method Promise forgotPasswordAsync( array $args = [] )
 * @method Result getCSVHeader( array $args = [] )
 * @method Promise getCSVHeaderAsync( array $args = [] )
 * @method Result getDevice( array $args = [] )
 * @method Promise getDeviceAsync( array $args = [] )
 * @method Result getGroup( array $args = [] )
 * @method Promise getGroupAsync( array $args = [] )
 * @method Result getIdentityProviderByIdentifier( array $args = [] )
 * @method Promise getIdentityProviderByIdentifierAsync( array $args = [] )
 * @method Result getSigningCertificate( array $args = [] )
 * @method Promise getSigningCertificateAsync( array $args = [] )
 * @method Result getUICustomization( array $args = [] )
 * @method Promise getUICustomizationAsync( array $args = [] )
 * @method Result getUser( array $args = [] )
 * @method Promise getUserAsync( array $args = [] )
 * @method Result getUserAttributeVerificationCode( array $args = [] )
 * @method Promise getUserAttributeVerificationCodeAsync( array $args = [] )
 * @method Result getUserPoolMfaConfig( array $args = [] )
 * @method Promise getUserPoolMfaConfigAsync( array $args = [] )
 * @method Result globalSignOut( array $args = [] )
 * @method Promise globalSignOutAsync( array $args = [] )
 * @method Result initiateAuth( array $args = [] )
 * @method Promise initiateAuthAsync( array $args = [] )
 * @method Result listDevices( array $args = [] )
 * @method Promise listDevicesAsync( array $args = [] )
 * @method Result listGroups( array $args = [] )
 * @method Promise listGroupsAsync( array $args = [] )
 * @method Result listIdentityProviders( array $args = [] )
 * @method Promise listIdentityProvidersAsync( array $args = [] )
 * @method Result listResourceServers( array $args = [] )
 * @method Promise listResourceServersAsync( array $args = [] )
 * @method Result listTagsForResource( array $args = [] )
 * @method Promise listTagsForResourceAsync( array $args = [] )
 * @method Result listUserImportJobs( array $args = [] )
 * @method Promise listUserImportJobsAsync( array $args = [] )
 * @method Result listUserPoolClients( array $args = [] )
 * @method Promise listUserPoolClientsAsync( array $args = [] )
 * @method Result listUserPools( array $args = [] )
 * @method Promise listUserPoolsAsync( array $args = [] )
 * @method Result listUsers( array $args = [] )
 * @method Promise listUsersAsync( array $args = [] )
 * @method Result listUsersInGroup( array $args = [] )
 * @method Promise listUsersInGroupAsync( array $args = [] )
 * @method Result resendConfirmationCode( array $args = [] )
 * @method Promise resendConfirmationCodeAsync( array $args = [] )
 * @method Result respondToAuthChallenge( array $args = [] )
 * @method Promise respondToAuthChallengeAsync( array $args = [] )
 * @method Result setRiskConfiguration( array $args = [] )
 * @method Promise setRiskConfigurationAsync( array $args = [] )
 * @method Result setUICustomization( array $args = [] )
 * @method Promise setUICustomizationAsync( array $args = [] )
 * @method Result setUserMFAPreference( array $args = [] )
 * @method Promise setUserMFAPreferenceAsync( array $args = [] )
 * @method Result setUserPoolMfaConfig( array $args = [] )
 * @method Promise setUserPoolMfaConfigAsync( array $args = [] )
 * @method Result setUserSettings( array $args = [] )
 * @method Promise setUserSettingsAsync( array $args = [] )
 * @method Result signUp( array $args = [] )
 * @method Promise signUpAsync( array $args = [] )
 * @method Result startUserImportJob( array $args = [] )
 * @method Promise startUserImportJobAsync( array $args = [] )
 * @method Result stopUserImportJob( array $args = [] )
 * @method Promise stopUserImportJobAsync( array $args = [] )
 * @method Result tagResource( array $args = [] )
 * @method Promise tagResourceAsync( array $args = [] )
 * @method Result untagResource( array $args = [] )
 * @method Promise untagResourceAsync( array $args = [] )
 * @method Result updateAuthEventFeedback( array $args = [] )
 * @method Promise updateAuthEventFeedbackAsync( array $args = [] )
 * @method Result updateDeviceStatus( array $args = [] )
 * @method Promise updateDeviceStatusAsync( array $args = [] )
 * @method Result updateGroup( array $args = [] )
 * @method Promise updateGroupAsync( array $args = [] )
 * @method Result updateIdentityProvider( array $args = [] )
 * @method Promise updateIdentityProviderAsync( array $args = [] )
 * @method Result updateResourceServer( array $args = [] )
 * @method Promise updateResourceServerAsync( array $args = [] )
 * @method Result updateUserAttributes( array $args = [] )
 * @method Promise updateUserAttributesAsync( array $args = [] )
 * @method Result updateUserPool( array $args = [] )
 * @method Promise updateUserPoolAsync( array $args = [] )
 * @method Result updateUserPoolClient( array $args = [] )
 * @method Promise updateUserPoolClientAsync( array $args = [] )
 * @method Result updateUserPoolDomain( array $args = [] )
 * @method Promise updateUserPoolDomainAsync( array $args = [] )
 * @method Result verifySoftwareToken( array $args = [] )
 * @method Promise verifySoftwareTokenAsync( array $args = [] )
 * @method Result verifyUserAttribute( array $args = [] )
 * @method Promise verifyUserAttributeAsync( array $args = [] )
 */
class CognitoIdentityProviderClient extends AwsClient {
}
