<?php

namespace Aws\StorageGateway;

use Aws\AwsClient;
use Aws\Result;
use GuzzleHttp\Promise\Promise;

/**
 * AWS Storage Gateway client.
 *
 * @method Result activateGateway( array $args = [] )
 * @method Promise activateGatewayAsync( array $args = [] )
 * @method Result addCache( array $args = [] )
 * @method Promise addCacheAsync( array $args = [] )
 * @method Result addTagsToResource( array $args = [] )
 * @method Promise addTagsToResourceAsync( array $args = [] )
 * @method Result addUploadBuffer( array $args = [] )
 * @method Promise addUploadBufferAsync( array $args = [] )
 * @method Result addWorkingStorage( array $args = [] )
 * @method Promise addWorkingStorageAsync( array $args = [] )
 * @method Result assignTapePool( array $args = [] )
 * @method Promise assignTapePoolAsync( array $args = [] )
 * @method Result attachVolume( array $args = [] )
 * @method Promise attachVolumeAsync( array $args = [] )
 * @method Result cancelArchival( array $args = [] )
 * @method Promise cancelArchivalAsync( array $args = [] )
 * @method Result cancelRetrieval( array $args = [] )
 * @method Promise cancelRetrievalAsync( array $args = [] )
 * @method Result createCachediSCSIVolume( array $args = [] )
 * @method Promise createCachediSCSIVolumeAsync( array $args = [] )
 * @method Result createNFSFileShare( array $args = [] )
 * @method Promise createNFSFileShareAsync( array $args = [] )
 * @method Result createSMBFileShare( array $args = [] )
 * @method Promise createSMBFileShareAsync( array $args = [] )
 * @method Result createSnapshot( array $args = [] )
 * @method Promise createSnapshotAsync( array $args = [] )
 * @method Result createSnapshotFromVolumeRecoveryPoint( array $args = [] )
 * @method Promise createSnapshotFromVolumeRecoveryPointAsync( array $args = [] )
 * @method Result createStorediSCSIVolume( array $args = [] )
 * @method Promise createStorediSCSIVolumeAsync( array $args = [] )
 * @method Result createTapeWithBarcode( array $args = [] )
 * @method Promise createTapeWithBarcodeAsync( array $args = [] )
 * @method Result createTapes( array $args = [] )
 * @method Promise createTapesAsync( array $args = [] )
 * @method Result deleteBandwidthRateLimit( array $args = [] )
 * @method Promise deleteBandwidthRateLimitAsync( array $args = [] )
 * @method Result deleteChapCredentials( array $args = [] )
 * @method Promise deleteChapCredentialsAsync( array $args = [] )
 * @method Result deleteFileShare( array $args = [] )
 * @method Promise deleteFileShareAsync( array $args = [] )
 * @method Result deleteGateway( array $args = [] )
 * @method Promise deleteGatewayAsync( array $args = [] )
 * @method Result deleteSnapshotSchedule( array $args = [] )
 * @method Promise deleteSnapshotScheduleAsync( array $args = [] )
 * @method Result deleteTape( array $args = [] )
 * @method Promise deleteTapeAsync( array $args = [] )
 * @method Result deleteTapeArchive( array $args = [] )
 * @method Promise deleteTapeArchiveAsync( array $args = [] )
 * @method Result deleteVolume( array $args = [] )
 * @method Promise deleteVolumeAsync( array $args = [] )
 * @method Result describeBandwidthRateLimit( array $args = [] )
 * @method Promise describeBandwidthRateLimitAsync( array $args = [] )
 * @method Result describeCache( array $args = [] )
 * @method Promise describeCacheAsync( array $args = [] )
 * @method Result describeCachediSCSIVolumes( array $args = [] )
 * @method Promise describeCachediSCSIVolumesAsync( array $args = [] )
 * @method Result describeChapCredentials( array $args = [] )
 * @method Promise describeChapCredentialsAsync( array $args = [] )
 * @method Result describeGatewayInformation( array $args = [] )
 * @method Promise describeGatewayInformationAsync( array $args = [] )
 * @method Result describeMaintenanceStartTime( array $args = [] )
 * @method Promise describeMaintenanceStartTimeAsync( array $args = [] )
 * @method Result describeNFSFileShares( array $args = [] )
 * @method Promise describeNFSFileSharesAsync( array $args = [] )
 * @method Result describeSMBFileShares( array $args = [] )
 * @method Promise describeSMBFileSharesAsync( array $args = [] )
 * @method Result describeSMBSettings( array $args = [] )
 * @method Promise describeSMBSettingsAsync( array $args = [] )
 * @method Result describeSnapshotSchedule( array $args = [] )
 * @method Promise describeSnapshotScheduleAsync( array $args = [] )
 * @method Result describeStorediSCSIVolumes( array $args = [] )
 * @method Promise describeStorediSCSIVolumesAsync( array $args = [] )
 * @method Result describeTapeArchives( array $args = [] )
 * @method Promise describeTapeArchivesAsync( array $args = [] )
 * @method Result describeTapeRecoveryPoints( array $args = [] )
 * @method Promise describeTapeRecoveryPointsAsync( array $args = [] )
 * @method Result describeTapes( array $args = [] )
 * @method Promise describeTapesAsync( array $args = [] )
 * @method Result describeUploadBuffer( array $args = [] )
 * @method Promise describeUploadBufferAsync( array $args = [] )
 * @method Result describeVTLDevices( array $args = [] )
 * @method Promise describeVTLDevicesAsync( array $args = [] )
 * @method Result describeWorkingStorage( array $args = [] )
 * @method Promise describeWorkingStorageAsync( array $args = [] )
 * @method Result detachVolume( array $args = [] )
 * @method Promise detachVolumeAsync( array $args = [] )
 * @method Result disableGateway( array $args = [] )
 * @method Promise disableGatewayAsync( array $args = [] )
 * @method Result joinDomain( array $args = [] )
 * @method Promise joinDomainAsync( array $args = [] )
 * @method Result listFileShares( array $args = [] )
 * @method Promise listFileSharesAsync( array $args = [] )
 * @method Result listGateways( array $args = [] )
 * @method Promise listGatewaysAsync( array $args = [] )
 * @method Result listLocalDisks( array $args = [] )
 * @method Promise listLocalDisksAsync( array $args = [] )
 * @method Result listTagsForResource( array $args = [] )
 * @method Promise listTagsForResourceAsync( array $args = [] )
 * @method Result listTapes( array $args = [] )
 * @method Promise listTapesAsync( array $args = [] )
 * @method Result listVolumeInitiators( array $args = [] )
 * @method Promise listVolumeInitiatorsAsync( array $args = [] )
 * @method Result listVolumeRecoveryPoints( array $args = [] )
 * @method Promise listVolumeRecoveryPointsAsync( array $args = [] )
 * @method Result listVolumes( array $args = [] )
 * @method Promise listVolumesAsync( array $args = [] )
 * @method Result notifyWhenUploaded( array $args = [] )
 * @method Promise notifyWhenUploadedAsync( array $args = [] )
 * @method Result refreshCache( array $args = [] )
 * @method Promise refreshCacheAsync( array $args = [] )
 * @method Result removeTagsFromResource( array $args = [] )
 * @method Promise removeTagsFromResourceAsync( array $args = [] )
 * @method Result resetCache( array $args = [] )
 * @method Promise resetCacheAsync( array $args = [] )
 * @method Result retrieveTapeArchive( array $args = [] )
 * @method Promise retrieveTapeArchiveAsync( array $args = [] )
 * @method Result retrieveTapeRecoveryPoint( array $args = [] )
 * @method Promise retrieveTapeRecoveryPointAsync( array $args = [] )
 * @method Result setLocalConsolePassword( array $args = [] )
 * @method Promise setLocalConsolePasswordAsync( array $args = [] )
 * @method Result setSMBGuestPassword( array $args = [] )
 * @method Promise setSMBGuestPasswordAsync( array $args = [] )
 * @method Result shutdownGateway( array $args = [] )
 * @method Promise shutdownGatewayAsync( array $args = [] )
 * @method Result startGateway( array $args = [] )
 * @method Promise startGatewayAsync( array $args = [] )
 * @method Result updateBandwidthRateLimit( array $args = [] )
 * @method Promise updateBandwidthRateLimitAsync( array $args = [] )
 * @method Result updateChapCredentials( array $args = [] )
 * @method Promise updateChapCredentialsAsync( array $args = [] )
 * @method Result updateGatewayInformation( array $args = [] )
 * @method Promise updateGatewayInformationAsync( array $args = [] )
 * @method Result updateGatewaySoftwareNow( array $args = [] )
 * @method Promise updateGatewaySoftwareNowAsync( array $args = [] )
 * @method Result updateMaintenanceStartTime( array $args = [] )
 * @method Promise updateMaintenanceStartTimeAsync( array $args = [] )
 * @method Result updateNFSFileShare( array $args = [] )
 * @method Promise updateNFSFileShareAsync( array $args = [] )
 * @method Result updateSMBFileShare( array $args = [] )
 * @method Promise updateSMBFileShareAsync( array $args = [] )
 * @method Result updateSMBSecurityStrategy( array $args = [] )
 * @method Promise updateSMBSecurityStrategyAsync( array $args = [] )
 * @method Result updateSnapshotSchedule( array $args = [] )
 * @method Promise updateSnapshotScheduleAsync( array $args = [] )
 * @method Result updateVTLDeviceType( array $args = [] )
 * @method Promise updateVTLDeviceTypeAsync( array $args = [] )
 */
class StorageGatewayClient extends AwsClient {
}
